unit UTrayIcon;
{************************************}
{        Copyright by -=Gott=-       }
{************************************}
interface
uses Windows,ShellAPI,Classes,Graphics,Menus,Messages,Controls;

const WM_TrayHandler=WM_User+1;
var NotifyIconData:TNotifyIconData;

type
  TClickType=(ctLClk,ctLDblClk,ctRClk,ctRDblClk);
  TTrayIcon=class(TComponent)
  private
    Showed:boolean;
    _Icon:TIcon;
    _Menu:TPopupMenu;
    _Hint:String;
    _MetPopup:TClickType;
    procedure TrayHandler(var Msg:TMessage);message WM_TrayHandler;
    procedure ShowPopup(Val:TClickType);
    function GetHint: string;
    procedure SetHint(const Value: string);
    procedure SetIcon(const Value: TIcon);
    function GetClick: TClickType;
    procedure SetClick(const Value: TClickType);
  public
    constructor Create(AOwner:TComponent);override;
    procedure Free;
    function TrayShow:boolean;//   
    function TrayHide:boolean;//   
    function TrayChangeIcon(Icon:TIcon):boolean;// 
    function TrayChangeHint(Val:string):boolean;// 
  published
    //    
    property ShowPopupOn:TClickType read GetClick write SetClick;
    property Hint:string read GetHint write SetHint;//  
    property MyMenu:TPopupMenu read _Menu write _Menu;//
    property TrayIcon:TIcon read _Icon write SetIcon;//  
  end;

procedure Register;
implementation

uses SysUtils;

procedure Register;
begin
  RegisterComponents('Samples',[TTrayIcon]);
end;

{ TTrayIcon }

constructor TTrayIcon.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  _Icon:=TIcon.Create;
  Showed:=false;
end;

procedure TTrayIcon.Free;
begin
  _Icon.Free;
  inherited Free;
end;

function TTrayIcon.GetClick: TClickType;
begin
  Result:=_MetPopup;
end;

function TTrayIcon.GetHint: string;
begin
  Result:=_Hint;
end;

procedure TTrayIcon.SetClick(const Value: TClickType);
begin
  _MetPopup:=Value;
end;

procedure TTrayIcon.SetHint(const Value: string);
begin
  _Hint:=Value;
end;

procedure TTrayIcon.SetIcon(const Value: TIcon);
begin
  _Icon.Assign(Value);
end;

procedure TTrayIcon.ShowPopup;
var tmp:TPoint;
begin
  if not Assigned(_Menu) then exit;
  if Val<>_MetPopup then exit;
  GetCursorPos(tmp);
  if Owner is TWinControl then
  SetForegroundWindow((Owner as TWinControl).Handle);
  _Menu.Popup(tmp.X,tmp.Y);
  if Owner is TWinControl then
  PostMessage((Owner as TWinControl).Handle,WM_USER,0,0);
end;

function TTrayIcon.TrayChangeHint(Val: string): boolean;
begin
  Result:=false;
  if not Showed then exit;
  SetHint(Val);
  StrCopy(NotifyIconData.szTip,PChar(Val));
  Result:=Shell_NotifyIcon(NIM_MODIFY,@NotifyIconData);
end;

function TTrayIcon.TrayChangeIcon(Icon: TIcon): boolean;
begin
  Result:=false;
  if not Showed then exit;
  SetIcon(Icon);
  NotifyIconData.hIcon:=Icon.Handle;
  Result:=Shell_NotifyIcon(NIM_MODIFY,@NotifyIconData);
end;

procedure TTrayIcon.TrayHandler(var Msg: TMessage);
begin
  case Msg.LParam of
  WM_LBUTTONUP:ShowPopup(ctLClk);
  WM_LBUTTONDBLCLK:ShowPopup(ctLDblClk);
  WM_RBUTTONUP:ShowPopup(ctRClk);
  WM_RBUTTONDBLCLK:ShowPopup(ctRDblClk);
  end;
end;

function TTrayIcon.TrayHide: boolean;
begin
  Result:=Shell_NotifyIcon(NIM_DELETE,@NotifyIconData);
  Showed:=false;
end;

function TTrayIcon.TrayShow: boolean;
begin
  if Showed then begin
    Result:=true;
    exit;
  end;
  with NotifyIconData do begin
    cbSize:=sizeof(NotifyIconData);
    Wnd:=AllocateHWnd(TrayHandler);
    uID:=0;
    uFlags:= NIF_MESSAGE or NIF_ICON or NIF_TIP;
    uCallbackMessage:=WM_TrayHandler;
    if Assigned(_Icon) then hIcon:=_Icon.Handle
    else hIcon:=LoadIcon(Hinstance,'MAINICON');
    StrCopy(szTip,PChar(_Hint));
  end;
  Result:=Shell_NotifyIcon(NIM_ADD,@NotifyIconData);
  Showed:=Result;
end;

end.
